/****************************************************************************/
/****************************************************************************/
import { DatasetEntry } from './interfaces/DatasetEntry';
import { DatasetMemberEntry } from './interfaces/DatasetMemberEntry';
import { Job } from './interfaces/Job';
import { LoadLibMemberEntry } from './interfaces/LoadLibMemberEntry';
import { SpoolFile } from './interfaces/SpoolFile';
import { USSEntry } from './interfaces/USSEntry';
export declare const USS_FILE_MODES_REX: RegExp;
/**
 * Parse a list of dataset list strings.
 *
 * @param lines - String list
 * @param migrationMode - True, in the case of migration mode
 * @returns Dateset entries
 */
export declare function parseDataSets(lines: string[], migrationMode: boolean): DatasetEntry[];
/**
 * Parse a list of PDS members like
 *
 * ```
 * Name     VV.MM   Created       Changed      Size  Init   Mod   Id
 * JVBR30   01.01 2018/09/07 2018/09/07 03:52    13    13     0  AAA
 * ```
 *
 * Meaning of each field:
 * name, version.modification, create date, modification date & time,
 * size in lines, size in lines at creation, lines modified, id of user who modified
 *
 * @param lines - String list
 * @param migrationMode - True, in the case of migration mode
 * @returns Dataset member entries
 */
export declare function parsePDSMembers(lines: string[], migrationMode: boolean): DatasetMemberEntry[];
export declare function parseLoadLibPDSMembers(lines: string[]): LoadLibMemberEntry[];
/**
 * Parse a list of USS directory list strings.
 *
 * @param lines - Lines
 * @param migrationMode - True, in the case of migration mode
 * @returns USS file entries
 */
export declare function parseUSSDirList(lines: string[], migrationMode: boolean): USSEntry[];
/**
 * Parses the spool file table.
 *
 * ```
 * ID  STEPNAME PROCSTEP C DDNAME   BYTE-COUNT
 * 001 JES2              H JESMSGLG      1582
 * 002 JES2        N/A   H JESJCL         324
 * 003 JES2        N/A   H JESYSMSG       978
 * ```
 * @param spoolFileTable - Spool file table
 * @param migrationMode - True, in the case of migration mode
 * @returns Spool files
 */
export declare function parseSpoolTable(spoolFileTable: string[], migrationMode: boolean): SpoolFile[];
/**
 * Parses job line.
 *
 * @param line - Job line
 * @returns Job object or undefined
 */
export declare function parseJobLine(line: string): Job | undefined;
/**
 * Parses job lines.
 *
 * @param lines - Job lines
 * @returns Job object array
 */
export declare function parseJobList(lines: string[]): Job[];
