"use strict";
/****************************************************************************/
/*                                                                          */
/* Copyright (c) 2017, 2023 IBM Corp.                                       */
/* All rights reserved. This program and the accompanying materials         */
/* are made available under the terms of the Eclipse Public License v1.0    */
/* which accompanies this distribution, and is available at                 */
/* http://www.eclipse.org/legal/epl-v10.html                                */
/*                                                                          */
/* Contributors:                                                            */
/*  IBM Corp. - initial API and implementation                              */
/*                                                                          */
/****************************************************************************/
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var parser_1 = __importDefault(require("ftp4/lib/parser"));
parser_1.default.parseListEntry = function (line) {
    return line;
};
var ftp4_1 = __importDefault(require("ftp4"));
var path_1 = __importDefault(require("path"));
var q_1 = __importDefault(require("q"));
var parser_2 = require("./parser");
var utils_1 = require("./utils");
var USSEntry_1 = require("./interfaces/USSEntry");
/**
 * Data transfer mode.
 */
var TransferMode;
(function (TransferMode) {
    /**
     * The data is transferred in text mode, in which encoding conversion like ASCII/EBCDIC will happen.
     */
    TransferMode["ASCII"] = "ascii";
    /**
     * The data is transferred in text mode, in which encoding conversion like ASCII/EBCDIC will happen, and
     * the EOL of each record in dataset will be removed.
     */
    TransferMode["ASCII_STRIP_EOL"] = "ascii_strip_eol";
    /**
     * The data is transferred in text mode, in which the trailing blanks will be removed.
     */
    TransferMode["ASCII_NO_TRAILING_BLANKS"] = "ascii_no_trailing_blanks";
    /**
     * The data is transferred in binary mode, in which no encoding conversion will happen.
     */
    TransferMode["BINARY"] = "binary";
    /**
     * The data is transferred in rdw mode, in which variable length dataset downloading is available.
     */
    TransferMode["ASCII_RDW"] = "ascii_rdw";
    /**
     * The data is transferred in rdw mode, in which variable length dataset downloading is available.
     */
    TransferMode["BINARY_RDW"] = "binary_rdw";
})(TransferMode = exports.TransferMode || (exports.TransferMode = {}));
/**
 * Job status result.
 */
var JobStatusResult;
(function (JobStatusResult) {
    JobStatusResult["SUCCESS"] = "success";
    JobStatusResult["ACTIVE"] = "active";
    JobStatusResult["FAIL"] = "fail";
    JobStatusResult["WAITING"] = "waiting";
    JobStatusResult["NOT_FOUND"] = "not found";
})(JobStatusResult = exports.JobStatusResult || (exports.JobStatusResult = {}));
/**
 * USS file type to operate.
 */
var FileToOperate;
(function (FileToOperate) {
    /**
     * Operate against one file or directory
     */
    FileToOperate[FileToOperate["FILE_OR_DIRECTORY"] = 0] = "FILE_OR_DIRECTORY";
    /**
     * Operate against the whole directory, including any file/directory under it.
     */
    FileToOperate[FileToOperate["WHOLE_DIRECTORY"] = 1] = "WHOLE_DIRECTORY";
})(FileToOperate = exports.FileToOperate || (exports.FileToOperate = {}));
/**
 * The max buffer size for downloading.
 */
var MAX_BUFFER_SIZE = 4 * 1024 * 1024;
var ZosAccessor = /** @class */ (function () {
    function ZosAccessor() {
        // Maximum allowed buffer size in one request.
        this.maxBufferSize = MAX_BUFFER_SIZE;
        /**
         * Indicates zos-node-accessor in migration mode.
         */
        this.migrationMode = false;
        this.client = new ftp4_1.default();
        this.connected = false;
    }
    /**
     * Set the max buffer size for downloading.
     *
     * @param maxBufferSize - Max buffer size in byte
     */
    ZosAccessor.prototype.setMaxBufferSize = function (maxBufferSize) {
        this.maxBufferSize = maxBufferSize;
    };
    /**
     * Returns the max buffer size for downloading.
     *
     * @returns Max buffer size for downloading
     */
    ZosAccessor.prototype.getMaxBufferSize = function () {
        return this.maxBufferSize;
    };
    /**
     * Sets zos-node-accessor to migration mode, which keeps the original key/value pair field in
     * `DatasetEntry`, 'DatasetMemberEntry`, `UssEntry`, and `SpoolFile`, so that it makes the
     * migration from v1.0.x to v2.x easy.
     *
     * @param migrationMode - True to enable migration mode
     */
    ZosAccessor.prototype.setMigrationMode = function (migrationMode) {
        this.migrationMode = migrationMode;
    };
    /**
     * Returns true if it's connected to z/OS FTP service successfully.
     *
     * @returns True if it's connected to z/OS FTP service successfully.
     */
    ZosAccessor.prototype.isConnected = function () {
        return this.connected;
    };
    /**
     * Connects to z/OS FTP service.
     *
     * @param option - The option for connecting z/OS FTP service
     * @returns This ZosAccessor object if success
     */
    ZosAccessor.prototype.connect = function (option) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpOption;
            var _this = this;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                this.username = option.user;
                this.client.on('ready', function () {
                    _this.connected = true;
                    deferred.resolve(_this);
                }).on('error', function (err) {
                    _this.connected = false;
                    deferred.reject(err);
                }).on('close', function () {
                    _this.connected = false;
                });
                ftpOption = {
                    host: option.host,
                    password: option.password,
                    port: option.port || 21,
                    user: option.user,
                    // tslint:disable-next-line: object-literal-sort-keys
                    secure: option.secure || false,
                    secureOptions: option.secureOptions,
                    connTimeout: option.connTimeout || 30000,
                    pasvTimeout: option.pasvTimeout || 30000,
                    keepalive: option.keepalive || 10000,
                };
                this.client.connect(ftpOption);
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Closes the connection to z/OS FTP service.
     */
    ZosAccessor.prototype.close = function () {
        return __awaiter(this, void 0, void 0, function () {
            var deferred;
            var _this = this;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                this.client.logout(function (err) {
                    if (err) {
                        deferred.reject(err);
                        return;
                    }
                    _this.client.end();
                    deferred.resolve();
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Submits SITE commands.
     *
     * @param siteCommands site commands separated with space
     * @returns what's returned from server
     */
    ZosAccessor.prototype.site = function (siteCommands) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                this.client.site(siteCommands, function (err, text) {
                    if (err) {
                        deferred.reject(err);
                        return;
                    }
                    deferred.resolve(text);
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Submits STAT command to query server status.
     *
     * @param option optional option. If not specified, query for all status.
     * @returns what's returned from server
     */
    ZosAccessor.prototype.stat = function (option) {
        return __awaiter(this, void 0, void 0, function () {
            var command;
            return __generator(this, function (_a) {
                command = option ? "stat (" + option : 'stat';
                return [2 /*return*/, this.send(command)];
            });
        });
    };
    /**
     * Sends any command that FTP can understand with SEND.
     *
     * @param command command to send
     * @returns what's returned from server
     */
    ZosAccessor.prototype.send = function (command) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                this.client._send(command, function (err, text) {
                    if (err) {
                        deferred.reject(err);
                        return;
                    }
                    deferred.resolve(text);
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Makes USS directory with the given directory name.
     *
     * @param directoryName - Directory name
     * @returns Promise to return void
     */
    ZosAccessor.prototype.makeDirectory = function (directoryName) {
        return __awaiter(this, void 0, void 0, function () {
            var ftpClient, deferred;
            return __generator(this, function (_a) {
                ftpClient = this.client;
                deferred = q_1.default.defer();
                ftpClient.mkdir(directoryName, function (err) {
                    if (err) {
                        deferred.reject(err);
                    }
                    else {
                        deferred.resolve();
                    }
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Allocates the sequential or partition (with the DCB attribut "PDSTYPE=PDS") dataset.
     * These attributes are transferred as FTP site sub commands. The tested attributes includes:
     *
     * ```
     * BLKsize/BLOCKSIze, BLocks, CYlinders, Directory, LRecl, PDSTYPE, PRImary, RECfm, SECondary, and TRacks
     * ```
     *
     * @param datasetName - Name of the dataset to allocate
     * @param allocateParamsOrString - String of space separated DCB attributes or the object of DCB attribute
     *            key-value pairs. eg. "LRECL=80 RECFM=FB" or {"LRECL": 80, "RECFM": "FB"}.
     * @returns Promise to return void
     */
    ZosAccessor.prototype.allocateDataset = function (datasetName, allocateParamsOrString) {
        return __awaiter(this, void 0, void 0, function () {
            var ftpClient, datasetList, allocateParamString, deferred_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        ftpClient = this.client;
                        datasetName = utils_1.Utils.ensureFullQualifiedDSN(datasetName);
                        return [4 /*yield*/, this.listDatasets(datasetName)];
                    case 1:
                        datasetList = _a.sent();
                        if (datasetList.length) {
                            throw new Error('Dataset ' + datasetName + ' already exists');
                        }
                        if (typeof allocateParamsOrString === 'string') {
                            allocateParamString = allocateParamsOrString;
                        }
                        else {
                            allocateParamString = this.allocateParamsToString(allocateParamsOrString);
                        }
                        if (allocateParamString.indexOf('DSORG=PO') !== -1 ||
                            allocateParamString.indexOf('PDSTYPE=PDS') !== -1 ||
                            allocateParamString.indexOf('PDSTYPE=PDSE') !== -1) {
                            // Remove DSORG since it's not valid site sub command.
                            allocateParamString = allocateParamString.replace('DSORG=PO', '');
                            deferred_1 = q_1.default.defer();
                            ftpClient.site(allocateParamString, function (err) {
                                if (err) {
                                    return deferred_1.reject(err);
                                }
                                ftpClient.mkdir(datasetName, function (err2) {
                                    if (err2) {
                                        deferred_1.reject(err2);
                                    }
                                    else {
                                        deferred_1.resolve();
                                    }
                                });
                            });
                            return [2 /*return*/, deferred_1.promise];
                        }
                        // Allocate a seq dataSet
                        // Upload a space here, because an empty Buffer may cause the "connection reset" error.
                        return [2 /*return*/, this.uploadDataset(Buffer.from(' '), datasetName, undefined, allocateParamString)];
                }
            });
        });
    };
    ZosAccessor.prototype.upload = function (input, destDataset, transferMode, allocateParamsOrString) {
        if (transferMode === void 0) { transferMode = TransferMode.ASCII; }
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpClient, allocateParamString, siteParams;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                ftpClient = this.client;
                // We still need this check in case the other TransferMode will be added in future.
                if (transferMode !== TransferMode.ASCII &&
                    transferMode !== TransferMode.BINARY) {
                    throw new Error('Unsupported data type: ' + transferMode);
                }
                if (typeof allocateParamsOrString === 'string') {
                    allocateParamString = allocateParamsOrString;
                }
                else {
                    allocateParamString = this.allocateParamsToString(allocateParamsOrString);
                }
                siteParams = ['FILETYPE=SEQ'];
                siteParams.push(allocateParamString);
                // ftpClient.ascii() or ftpClient.binary()
                ftpClient[transferMode](function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    ftpClient.site(siteParams.join(' '), function (err1) {
                        if (err1) {
                            return deferred.reject(err);
                        }
                        destDataset = utils_1.Utils.ensureFullQualifiedDSN(destDataset);
                        ftpClient.put(input, destDataset, function (err2) {
                            if (err2) {
                                deferred.reject(err2);
                            }
                            else {
                                deferred.resolve();
                            }
                        });
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Uploads data to the specified dataset on z/OS. The tested attributes for `allocateParamsOrString` includes:
     * These attributes are transferred as FTP site sub commands. The tested attributes includes:
     *
     * ```
     * BLKsize/BLOCKSIze, BLocks, CYlinders, Directory, LRecl, PDSTYPE, PRImary, RECfm, SECondary, and TRacks
     * ```
     *
     * @param input - Input, which can be a ReadableStream, a Buffer, or a path to a local file.
     * @param destDataset - Name of the dataset to store the uploaded data.
     * @param transferMode - Data transfer mode, either TransferMode.ASCII or TransferMode.BINARY.
     *            When transfering 'ascii' files, the end-of-line sequence of input should always be '\r\n'.
     *            Otherwise the transfered file will get truncated.
     * @param allocateParamsOrString - A string of space separated DCB attributes or an object of DCB attribute
     *          key-value pairs. eg. "LRECL=80 RECFM=FB" or {"LRECL": 80, "RECFM": "FB"}.
     */
    ZosAccessor.prototype.uploadDataset = function (input, destDataset, transferMode, allocateParamsOrString) {
        if (transferMode === void 0) { transferMode = TransferMode.ASCII; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.upload(input, destDataset, transferMode, allocateParamsOrString)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Upload data to the specified USS file on z/OS.
     *
     * @param input - Input, which can be a ReadableStream, a Buffer, or a path to a local file.
     * @param destFilePath - Path name of the destination file on z/OS.
     * @param transferMode - Data transfer mode, either TransferMode.ASCII or TransferMode.BINARY
     */
    ZosAccessor.prototype.uploadFile = function (input, destFilePath, transferMode) {
        if (transferMode === void 0) { transferMode = TransferMode.ASCII; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.upload(input, destFilePath, transferMode)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    ZosAccessor.prototype.download = function (dsn, transferMode, stream, siteParams) {
        if (transferMode === void 0) { transferMode = TransferMode.ASCII; }
        if (stream === void 0) { stream = false; }
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpClient, ftpCommand, sbsendeol, trailingBlanks, site;
            var _this = this;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                ftpClient = this.client;
                // We still need this check in case the other TransferMode will be added in future.
                if (transferMode !== TransferMode.ASCII &&
                    transferMode !== TransferMode.ASCII_NO_TRAILING_BLANKS &&
                    transferMode !== TransferMode.ASCII_STRIP_EOL &&
                    transferMode !== TransferMode.BINARY) {
                    throw new Error('Unsupported data type: ' + transferMode);
                }
                ftpCommand = ftpClient.binary.bind(ftpClient);
                if (transferMode === TransferMode.ASCII ||
                    transferMode === TransferMode.ASCII_NO_TRAILING_BLANKS ||
                    transferMode === TransferMode.ASCII_STRIP_EOL) {
                    ftpCommand = ftpClient.ascii.bind(ftpClient);
                }
                sbsendeol = 'SBSENDEOL=CRLF';
                if (transferMode === TransferMode.ASCII_STRIP_EOL) {
                    sbsendeol = 'SBSENDEOL=NONE';
                    transferMode = TransferMode.ASCII;
                }
                trailingBlanks = 'TRAILINGBLANKS';
                if (transferMode === TransferMode.ASCII_NO_TRAILING_BLANKS) {
                    trailingBlanks = '';
                }
                site = 'FILETYPE=SEQ ' + trailingBlanks + ' ' + sbsendeol;
                if (siteParams) {
                    site += ' ' + siteParams;
                }
                // ftpClient.ascii() or ftpClient.binary()
                ftpCommand(function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    ftpClient.site(site, function (err1) {
                        if (err1) {
                            return deferred.reject(err1);
                        }
                        dsn = utils_1.Utils.ensureFullQualifiedDSN(dsn);
                        ftpClient.get(dsn, function (err2, readStream) {
                            if (err2) {
                                return deferred.reject(err2);
                            }
                            if (stream) {
                                return deferred.resolve(readStream);
                            }
                            var chunks = [];
                            var bytes = 0;
                            readStream.on('data', function (chunk) {
                                if (bytes + chunk.length >= _this.maxBufferSize) {
                                    var message = "Reach the maximum buffer size, " + _this.maxBufferSize + " bytes. " +
                                        "Change to use stream mode for big file.";
                                    deferred.reject(new Error(message));
                                }
                                chunks.push(chunk);
                                bytes += chunk.length;
                            });
                            readStream.on('end', function () {
                                deferred.resolve(Buffer.concat(chunks));
                            });
                            readStream.on('error', function (err3) {
                                deferred.reject(err3);
                            });
                            readStream.resume();
                        });
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Downloads the specified dataset or member of patition dataset.
     *
     * @param dsn - Dataset name
     * @param transferMode - TransferMode.ASCII, TransferMode.ASCII_RDW, TransferMode.ASCII_STRIP_EOL,
     *                       TransferMode.ASCII_NO_TRAILING_BLANKS, TransferMode.BINARY, or TransferMode.BINARY_RDW.
     *                       The `TransferMode.ASCII`, ` TransferMode.ASCII_RDW`, `TransferMode.ASCII_STRIP_EOL`,
     *                       or TransferMode.ASCII_NO_TRAILING_BLANKS asks z/OS FTP service to
     *                       convert `EBCDIC` characters to `ASCII`. `TransferMode.ASCII_STRIP_EOL` asks z/OS FTP
     *                       service not to append a `CLRF` to the end of each record. The
     *                       `TransferMode.ASCII_NO_TRAILING_BLANKS` asks z/OS FTP service to remove trailing blanks.
     *                       `TransferMode.ASCII_RDW` and
     *                       `TransferMode.BINARY_RDW` support to download variable length dataset, which add 4-byte
     *                       Record Description Word (RDW) at the beginning of each record.
     * @param {boolean} stream - `true` if you want to obtain a `ReadStream` of the data set content, or `false`
     *                           to read a full dataset into memory (in Buffer). The buffer accepts up to 4MB data.
     *                           For large dataset, use `stream=true` instead.
     * @param {string} siteParams - Add extra site parameters, for example, 'sbd=(IBM-1047,ISO8859-1)' for encoding
     *                              setting.
     * @returns `ReadStream` or `Buffer`
     */
    ZosAccessor.prototype.downloadDataset = function (dsn, transferMode, stream, siteParams) {
        if (transferMode === void 0) { transferMode = TransferMode.ASCII; }
        if (stream === void 0) { stream = false; }
        return __awaiter(this, void 0, void 0, function () {
            var deferred_2, ftpClient;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (transferMode === TransferMode.ASCII_RDW || transferMode === TransferMode.BINARY_RDW) {
                            if (transferMode === TransferMode.ASCII_RDW) {
                                transferMode = TransferMode.ASCII;
                            }
                            else {
                                transferMode = TransferMode.BINARY;
                            }
                            deferred_2 = q_1.default.defer();
                            ftpClient = this.client;
                            ftpClient.site('rdw', function (err) { return __awaiter(_this, void 0, void 0, function () {
                                var file;
                                return __generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            if (err) {
                                                return [2 /*return*/, deferred_2.reject(err)];
                                            }
                                            return [4 /*yield*/, this.download(dsn, transferMode, stream, siteParams)];
                                        case 1:
                                            file = _a.sent();
                                            deferred_2.resolve(file);
                                            return [2 /*return*/];
                                    }
                                });
                            }); });
                            return [2 /*return*/, deferred_2.promise];
                        }
                        return [4 /*yield*/, this.download(dsn, transferMode, stream, siteParams)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Downloads the specified USS file.
     *
     * @param filePath - USS file path name
     * @param transferMode - TransferMode.ASCII, TransferMode.BINARY
     *                       When downloading a text dataset, transferMode should be either `TransferMode.ASCII`
     *                       so that z/OS FTP service converts `EBCDIC` characters to `ASCII`.
     * @param {boolean} stream - `true` if you want to obtain a `ReadStream` of the data set content, or `false`
     *                           to read a full dataset into memory (in Buffer). The buffer accepts up to 4MB data.
     *                           For large dataset, use `stream=true` instead.
     * @returns `ReadStream` or `Buffer`
     */
    ZosAccessor.prototype.downloadFile = function (filePath, transferMode, stream) {
        if (transferMode === void 0) { transferMode = TransferMode.ASCII; }
        if (stream === void 0) { stream = false; }
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.download(filePath, transferMode, stream)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    ZosAccessor.prototype.list = function (dsn) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpClient;
            var _this = this;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                ftpClient = this.client;
                ftpClient.site('FILETYPE=SEQ ISPFSTATS', function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    dsn = utils_1.Utils.ensureFullQualifiedDSN(dsn);
                    ftpClient.list(dsn, function (err1, list) {
                        if (err1) {
                            if (err1.code === 550 && list === undefined && /No [\s\w]+ found/.test(err1.message)) {
                                // "No data sets found" or "No members found"
                                deferred.resolve([]);
                            }
                            else {
                                deferred.reject(err1);
                            }
                        }
                        else {
                            if (list[0].match(/^total/)) {
                                list.shift(); // Pop header
                                deferred.resolve(parser_2.parseUSSDirList(list, _this.migrationMode));
                            }
                            else if (list[0].indexOf('Volume') >= 0 && list[0].indexOf('Dsname') >= 0) {
                                deferred.resolve(parser_2.parseDataSets(list, _this.migrationMode));
                            }
                            else if (list[0].indexOf('Name') >= 0 && list[0].indexOf('Id') >= 0) {
                                deferred.resolve(parser_2.parsePDSMembers(list, _this.migrationMode));
                            }
                            else if (list[0].indexOf('Name') >= 0 && list[0].indexOf('Amode') >= 0) {
                                deferred.resolve(parser_2.parseLoadLibPDSMembers(list));
                            }
                            else if (list[0].indexOf(' ') !== -1) {
                                // The following line is returned when listing a regular file or symbolic link.
                                // lrwxrwxrwx   1 USER  GROUP         12 Jul 13  2017 /tmp -> $SYSNAME/tmp
                                // -rw-------   1 USER  GROUP    2152185 Nov  7 20:40 /tmp/abc.txt
                                var items = list[0].split(' ');
                                if (parser_2.USS_FILE_MODES_REX.test(items[0])) {
                                    // The first item must contain the char symbol of file mode, dir, link only.
                                    deferred.resolve(parser_2.parseUSSDirList(list, _this.migrationMode));
                                }
                                else {
                                    deferred.resolve([]);
                                }
                            }
                            else {
                                deferred.reject(new Error('Unrecognized file list header: ' + list[0]));
                            }
                        }
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Lists the datasets whose names match with the given dataset name.
     *
     * @param dsn - Full qualified name of dataset, or datadata name with wildcards (* or ?)
     * @returns Dataset entries
     */
    ZosAccessor.prototype.listDatasets = function (dsn) {
        return __awaiter(this, void 0, void 0, function () {
            var entries;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.list(dsn)];
                    case 1:
                        entries = _a.sent();
                        return [2 /*return*/, entries.map(function (entry) { return entry; })];
                }
            });
        });
    };
    /**
     * Lists the members of partition dataset.
     *
     * @param partitionDsn - Full qualified name of patition dataset
     * @returns Dataset member entries
     */
    ZosAccessor.prototype.listMembers = function (partitionDsn) {
        return __awaiter(this, void 0, void 0, function () {
            var entries;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.list(utils_1.Utils.removeQuote(partitionDsn) + "(*)")];
                    case 1:
                        entries = _a.sent();
                        return [2 /*return*/, entries.map(function (entry) { return entry; })];
                }
            });
        });
    };
    /**
     * Lists files whose names match with the given path name.
     *
     * @param dirPath - Directory to list or file name with widecards (* or ?)
     * @returns USS file entries
     */
    ZosAccessor.prototype.listFiles = function (dirPath) {
        return __awaiter(this, void 0, void 0, function () {
            var entries;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.list(dirPath)];
                    case 1:
                        entries = _a.sent();
                        return [2 /*return*/, entries.map(function (entry) { return entry; })];
                }
            });
        });
    };
    /**
     * Deletes the dataset or member of parition datate whose names match with the given dataset name.
     *
     * @param dsn - Full qualified name of dataset. The datadata name with wildcards (* or ?) is not supported.
     */
    ZosAccessor.prototype.deleteDataset = function (dsn) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred, dataset, ftpClient;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                dataset = utils_1.Utils.ensureFullQualifiedDSN(dsn);
                ftpClient = this.client;
                ftpClient.site('FILETYPE=SEQ', function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    ftpClient.delete(dataset, function (err1) {
                        if (err1) {
                            deferred.reject(err1);
                        }
                        else {
                            deferred.resolve();
                        }
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Deletes the USS files or directory whose names match with the given file path.
     *
     * @param filePath - The path name of USS file or directory. The path name with wildcards (* or ?) is not supported.
     * @param fileType - Either FILE_OR_DIRECTORY or WHOLE_DIRECTORY
     */
    ZosAccessor.prototype.deleteFile = function (filePath, fileType) {
        if (fileType === void 0) { fileType = FileToOperate.FILE_OR_DIRECTORY; }
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpClient;
            var _this = this;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                ftpClient = this.client;
                ftpClient.site('FILETYPE=SEQ', function (err) { return __awaiter(_this, void 0, void 0, function () {
                    var list, _i, list_1, item, entry;
                    return __generator(this, function (_a) {
                        switch (_a.label) {
                            case 0:
                                if (!err) return [3 /*break*/, 1];
                                return [2 /*return*/, deferred.reject(err)];
                            case 1:
                                if (!(fileType === FileToOperate.WHOLE_DIRECTORY)) return [3 /*break*/, 10];
                                return [4 /*yield*/, this.listFiles(filePath)];
                            case 2:
                                list = _a.sent();
                                _i = 0, list_1 = list;
                                _a.label = 3;
                            case 3:
                                if (!(_i < list_1.length)) return [3 /*break*/, 8];
                                item = list_1[_i];
                                entry = item;
                                if (!(entry.fileType === USSEntry_1.FileType.DIRECTORY)) return [3 /*break*/, 5];
                                return [4 /*yield*/, this.deleteFile(path_1.default.join(filePath, entry.name), FileToOperate.WHOLE_DIRECTORY)];
                            case 4:
                                _a.sent();
                                return [3 /*break*/, 7];
                            case 5: return [4 /*yield*/, this.deleteFile(path_1.default.join(filePath, entry.name), FileToOperate.FILE_OR_DIRECTORY)];
                            case 6:
                                _a.sent();
                                _a.label = 7;
                            case 7:
                                _i++;
                                return [3 /*break*/, 3];
                            case 8: return [4 /*yield*/, this.deleteFile(filePath, FileToOperate.FILE_OR_DIRECTORY)];
                            case 9:
                                _a.sent();
                                deferred.resolve();
                                return [3 /*break*/, 11];
                            case 10:
                                ftpClient.delete(filePath, function (err1) {
                                    if (err1) {
                                        deferred.reject(err1);
                                    }
                                    else {
                                        deferred.resolve();
                                    }
                                });
                                _a.label = 11;
                            case 11: return [2 /*return*/];
                        }
                    });
                }); });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    ZosAccessor.prototype.rename = function (name, newName) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred, oldFQD, newFQD, ftpClient;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                oldFQD = utils_1.Utils.ensureFullQualifiedDSN(name);
                newFQD = utils_1.Utils.ensureFullQualifiedDSN(newName);
                ftpClient = this.client;
                ftpClient.site('FILETYPE=SEQ', function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    ftpClient.rename(oldFQD, newFQD, function (err1) {
                        if (err1) {
                            deferred.reject(err1);
                        }
                        else {
                            deferred.resolve();
                        }
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Renames dataset, member in partition dataset.
     *
     * @param dsn Current name
     * @param newDsn New name
     */
    ZosAccessor.prototype.renameDataset = function (dsn, newDsn) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.rename(dsn, newDsn)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Renames USS file/directory.
     *
     * @param name Current name
     * @param newName New name
     */
    ZosAccessor.prototype.renameFile = function (name, newName) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.rename(name, newName)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Lists the jobs matching the given query option. If the query option is not provided, it will list all jobs
     * of the current user.
     *
     * @param queryOption - Option which contains
     *                          jobName: Job name, which is optional and can contain a wildcard (*)
     *                          jobId:   Job ID, which is optional
     *                          owner:   Job owner, which is optional and can contain a wildcard (*)
     *                          status:  Job status, eg. ALL, OUTPUT, which is optional
     * @returns List of jobs
     */
    ZosAccessor.prototype.listJobs = function (queryOption) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpClient, option;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                ftpClient = this.client;
                option = {
                    jobName: '',
                };
                if (queryOption === undefined) {
                    // Default queryOption is job name for compatibility.
                    option = { jobName: '*', owner: this.username, status: 'ALL' };
                }
                else {
                    this.checkJobNameAndOwner(queryOption.jobName, queryOption.owner);
                    option.jobName = queryOption.jobName || '*';
                    option.jobId = queryOption.jobId;
                    option.owner = queryOption.owner || this.username;
                    option.status = queryOption.status || 'ALL';
                }
                ftpClient.ascii(function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    var pathToList = option.jobId || '*';
                    var siteParams = ['FILETYPE=JES'];
                    siteParams.push("JESJOBNAME=" + option.jobName);
                    siteParams.push("JESOWNER=" + option.owner);
                    siteParams.push("JESSTATUS=" + option.status);
                    // Issue command like "site FILETYPE=JES JESJOBNAME=H* JESOWNER=* JESSTATUS=ALL"
                    ftpClient.site(siteParams.join(' '), function (err1) {
                        if (err1) {
                            return deferred.reject(err1);
                        }
                        ftpClient.list(pathToList, function (err2, list) {
                            if (err2) {
                                if (err2.code === 550) {
                                    // No job is found.
                                    deferred.resolve([]);
                                }
                                else {
                                    deferred.reject(err2);
                                }
                            }
                            else {
                                deferred.resolve(parser_2.parseJobList(list));
                            }
                        });
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Returns the status the job identified by job id and optional job name.
     *
     * @param queryOption - Option which contains
     *                          jobId:   Job ID, which is required
     *                          owner:   Job owner, which is optional
     * @returns List of job status
     */
    ZosAccessor.prototype.queryJob = function (queryOption) {
        return __awaiter(this, void 0, void 0, function () {
            var option;
            var _this = this;
            return __generator(this, function (_a) {
                option = {
                    jobId: queryOption.jobId,
                    owner: queryOption.owner || this.username,
                };
                return [2 /*return*/, this.listJobs(option).then(function (jobs) {
                        var _loop_1 = function (job) {
                            if (job.jobId && queryOption.jobId && job.jobId.toUpperCase() === queryOption.jobId.toUpperCase()) {
                                if (job.status === 'INPUT') {
                                    return { value: JobStatusResult.WAITING };
                                }
                                else if (job.status === 'HELD') {
                                    return { value: JobStatusResult.FAIL };
                                }
                                else if (job.status === 'ACTIVE') {
                                    return { value: JobStatusResult.ACTIVE };
                                }
                                else if (job.status === 'OUTPUT') {
                                    if (job.extra && job.extra.indexOf('RC=0000') !== -1) {
                                        return { value: JobStatusResult.SUCCESS };
                                    }
                                    else if (job.extra && job.extra.indexOf('ABEND=') !== -1) {
                                        return { value: JobStatusResult.FAIL };
                                    }
                                    else if (job.extra && job.extra.indexOf('JCL error') !== -1) {
                                        return { value: JobStatusResult.FAIL };
                                    }
                                    else {
                                        var deferred_3 = q_1.default.defer();
                                        _this.getJobStatus({ jobId: queryOption.jobId, owner: option.owner })
                                            .then(function (jobStatus) {
                                            if (jobStatus.rc === 0) {
                                                deferred_3.resolve(JobStatusResult.SUCCESS);
                                            }
                                            else {
                                                deferred_3.resolve(JobStatusResult.FAIL);
                                            }
                                        })
                                            .catch(function (err) {
                                            deferred_3.reject(err);
                                        });
                                        return { value: deferred_3.promise };
                                    }
                                }
                                return { value: JobStatusResult.FAIL };
                            }
                        };
                        for (var _i = 0, jobs_1 = jobs; _i < jobs_1.length; _i++) {
                            var job = jobs_1[_i];
                            var state_1 = _loop_1(job);
                            if (typeof state_1 === "object")
                                return state_1.value;
                        }
                        return JobStatusResult.NOT_FOUND;
                    })];
            });
        });
    };
    /**
     * Returns the job RC by reading JESMSGLG.
     *
     * @param queryOption - Option which contains
     *                          jobId:   Job ID, which is required
     *                          owner:   Specify a JES job owner, which is optional and can contain a wildcard (*)
     * @returns RC number or error string
     */
    ZosAccessor.prototype.getRCFromJESMSGLG = function (queryOption) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.getJobLog(queryOption).then(function (log) {
                        var rc;
                        var logContents = log;
                        if (logContents) {
                            var EYE_CATCHER_1_1 = 'ENDED - RC=';
                            var EYE_CATCHER_2_1 = 'ENDED - ABEND=';
                            var EYE_CATCHER_3_1 = 'NOT AUTHORIZED';
                            logContents.split('\n').forEach(function (line) {
                                var posn1 = line.indexOf(EYE_CATCHER_1_1);
                                var posn2 = line.indexOf(EYE_CATCHER_2_1);
                                var posn3 = line.indexOf(EYE_CATCHER_3_1);
                                if (posn1 !== -1) {
                                    rc = parseInt(line.substring(posn1 + EYE_CATCHER_1_1.length).trim(), 10);
                                }
                                else if (posn2 !== -1) {
                                    rc = 'ABEND ' + line.substring(posn2 + EYE_CATCHER_2_1.length).trim();
                                }
                                else if (posn3 !== -1) {
                                    rc = 'SEC ERROR';
                                }
                            });
                        }
                        if (typeof (rc) === 'number') {
                            rc = parseInt(rc, 10);
                        }
                        return q_1.default.resolve(rc);
                    })];
            });
        });
    };
    /**
     * Returns the status of the job specified by query option. Example of the returned job status:
     *
     * ```
     *  {
     *      jobname: "HRECALLW",
     *      jobid: "JOB06385",
     *      owner: "USER",
     *      status: "OUTPUT",
     *      class: "A",
     *      rc: 0,
     *      spoolFiles: [
     *         {
     *          id: 2,
     *          stepName: "JES2",
     *          procStep: "N/A",
     *          class: "H",
     *          ddName: "JESJCL",
     *          byteCount: 315
     *        }
     *      ]
     * }
     * ```
     *
     * @param queryOption - Option which contains
     *                          jobId:   Job ID, which is required
     *                          owner:   Job owner, which is optional and can contain a wildcard (*)
     * @returns Job status
     */
    ZosAccessor.prototype.getJobStatus = function (queryOption) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpClient, option;
            var _this = this;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                ftpClient = this.client;
                option = {
                    jobId: queryOption.jobId,
                    owner: queryOption.owner || this.username,
                };
                if (option.jobId === undefined) {
                    throw new Error('The job ID is required.');
                }
                ftpClient.ascii(function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    // eslint-disable-next-line @typescript-eslint/no-unused-vars
                    ftpClient.site("FILETYPE=JES SBSENDEOL=CRLF JESJOBNAME=* JESOWNER=" + option.owner, function (err1) {
                        if (err1) {
                            return deferred.reject(err1);
                        }
                        ftpClient.list(option.jobId, function (err2, list) {
                            /* Expected list response
                            JOBNAME  JOBID    OWNER    STATUS CLASS
                            HRECALLW JOB02094 VPADEV   INPUT  A
        
                            or
        
                            JOBNAME  JOBID    OWNER    STATUS CLASS
                            HRECALLW JOB02094 VPADEV   ACTIVE A
                            --------
                                    STEPNAME HRECALL  PROCNAME        N/A
                                    CPUTIME     0.012 ELAPSED TIME    117.515
        
                            or
        
                            JOBNAME  JOBID    OWNER    STATUS CLASS
                            HRECALLW JOB02094 VPADEV   OUTPUT A        RC=0000
                            --------
                                    ID  STEPNAME PROCSTEP C DDNAME   BYTE-COUNT
                                    001 JES2        N/A   H JESMSGLG      1582
                                    002 JES2        N/A   H JESJCL         324
                                    003 JES2        N/A   H JESYSMSG       978
                            3 spool files
                            **/
                            if (err2) {
                                return deferred.reject(err2);
                            }
                            var jobDescIndex = list.findIndex(function (line) {
                                return /^\s*JOBNAME\s+/i.test(line);
                            });
                            if (jobDescIndex === -1) {
                                return deferred.reject(new Error('Cannot find job header line'));
                            }
                            var job = parser_2.parseJobLine(list[jobDescIndex + 1].trim());
                            var jobStatus = __assign({}, job);
                            if (jobStatus.extra) {
                                jobStatus.extra.split(/\s+/).forEach(function (entry) {
                                    var pair = entry.split('=');
                                    if (pair.length === 2) {
                                        if (pair[0].toUpperCase() === 'RC') {
                                            jobStatus.rc = parseInt(pair[1], 10);
                                            jobStatus.retcode = pair[0] + ' ' + pair[1];
                                        }
                                    }
                                });
                            }
                            // For TSO users
                            //  No proper data to cover this so far.
                            //
                            // const tsoFieldsIndex = list.findIndex((line: string) => {
                            //     return /^\s+STEPNAME\s+/i.test(line);
                            // });
                            // if (tsoFieldsIndex !== -1) {
                            //     const tsoFieldStr = list[tsoFieldsIndex] + ' ' + list[tsoFieldsIndex + 1];
                            //     const fields = tsoFieldStr.split(/\s+(?!TIME)/i);
                            //     for (let i = 0; i < fields.length; i += 2) {
                            //         jobStatus[fields[i].toLowerCase()] = fields[i + 1];
                            //     }
                            // }
                            // Spool files
                            var spoolFileIndex = list.findIndex(function (line) {
                                return /^\s+ID\s+/i.test(line);
                            });
                            if (spoolFileIndex !== -1) {
                                // Spool file will be available when job finished
                                jobStatus.spoolFiles = parser_2.parseSpoolTable(list.slice(spoolFileIndex), _this.migrationMode);
                            }
                            // USER  TSU18242 USER  OUTPUT TSU      ABEND=622 1 spool files
                            // USER  JOB00256 USER  OUTPUT A        (JCL error) 3 spool files
                            var extra = jobStatus.extra;
                            if (extra && (extra.includes('error'))) {
                                jobStatus.rc = extra;
                                jobStatus.retcode = 'JCL ERROR';
                                deferred.resolve(jobStatus);
                                return;
                            }
                            else if (extra && (extra.includes('ABEND='))) {
                                jobStatus.rc = extra;
                                jobStatus.retcode = extra.replace('=', ' ');
                                deferred.resolve(jobStatus);
                                return;
                            }
                            // If job finished, while FTP doesn't provide RC
                            if (jobStatus.status === 'OUTPUT' && (jobStatus.rc === undefined)) {
                                var spoolFiles = jobStatus.spoolFiles;
                                if (spoolFiles !== undefined) {
                                    // Read RC from JESMSGLG
                                    var file = spoolFiles.find(function (spoolFile) {
                                        return spoolFile.ddName === 'JESMSGLG';
                                    });
                                    if (file) {
                                        var optionForJESMSGLG = {
                                            fileId: file.id,
                                            jobId: option.jobId,
                                            owner: option.owner,
                                        };
                                        _this.getRCFromJESMSGLG(optionForJESMSGLG)
                                            .then(function (rc) {
                                            jobStatus.rc = rc;
                                            if (typeof (rc) === 'number' && isNaN(rc) === false) {
                                                jobStatus.retcode = 'RC ' + (Array(4).join('0') + rc).slice(-4);
                                            }
                                            else {
                                                jobStatus.retcode = rc.toString();
                                            }
                                            deferred.resolve(jobStatus);
                                        })
                                            .catch(function (err) {
                                            deferred.reject(err);
                                        });
                                    }
                                    else {
                                        deferred.resolve(jobStatus);
                                    }
                                }
                            }
                            else {
                                deferred.resolve(jobStatus);
                            }
                        });
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Returns job spool files identified by jobId.
     *
     * @param queryOption - Option which contains
     *                          jobId:   Job ID, which is required
     *                          fileId:  Spool file index (1, 2, 3...) or
     *                                      -1 - return all spool files joined with the `!! END OF JES SPOOL FILE !!`
     *                          owner:   Specify a JES job owner, which is optional and can contain a wildcard (*)
     * @return Job log file contents
     */
    ZosAccessor.prototype.getJobLog = function (queryOption) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpClient, option;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                ftpClient = this.client;
                option = {
                    fileId: queryOption.fileId || -1,
                    jobId: queryOption.jobId,
                    owner: queryOption.owner || this.username,
                };
                if (option.jobId === undefined) {
                    throw new Error('The job ID is required.');
                }
                ftpClient.ascii(function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    var siteParams = ['FILETYPE=JES'];
                    siteParams.push('SBSENDEOL=CRLF');
                    siteParams.push("JESJOBNAME=*");
                    siteParams.push("JESOWNER=" + option.owner);
                    var site = siteParams.join(' ');
                    ftpClient.site(site, function (err1) {
                        if (err1) {
                            return deferred.reject(err1);
                        }
                        var jobLogID = option.fileId === -1 ? option.jobId + ".x" : option.jobId + "." + option.fileId;
                        ftpClient.get(jobLogID, function (err2, readStream) {
                            if (err2) {
                                return deferred.reject(err2);
                            }
                            var chunks = [];
                            readStream.on('data', function (chunk) {
                                chunks.push(chunk);
                            });
                            readStream.on('end', function () {
                                var logString = Buffer.concat(chunks).toString('utf8');
                                return deferred.resolve(logString);
                            });
                            readStream.on('error', function (err3) {
                                deferred.reject(err3);
                            });
                            readStream.resume();
                        });
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Deletes the job of the specified job id.
     *
     * @param queryOption - Option which contains
     *                          jobId:   Job ID, which is required
     *                          owner:   Job owner, which is optional and can contain a wildcard (*)
     */
    ZosAccessor.prototype.deleteJob = function (queryOption) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpClient, option;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                ftpClient = this.client;
                option = {
                    jobId: queryOption.jobId,
                    owner: queryOption.owner || this.username,
                };
                if (option.jobId === undefined) {
                    throw new Error('The job ID is required.');
                }
                ftpClient.site('FILETYPE=JES JESJOBNAME=*', function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    ftpClient.delete(option.jobId, function (err1) {
                        if (err1) {
                            deferred.reject(err1);
                        }
                        else {
                            deferred.resolve();
                        }
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Submits job with the specified JCL text.
     *
     * @param jclText JCL text to submit
     * @return Job ID
     */
    ZosAccessor.prototype.submitJCL = function (jclText) {
        return __awaiter(this, void 0, void 0, function () {
            var deferred, ftpClient;
            return __generator(this, function (_a) {
                deferred = q_1.default.defer();
                ftpClient = this.client;
                if (typeof jclText === 'string') {
                    // FTP server requires ascii text ends with \r\n
                    jclText = jclText.replace(/\r?\n/g, '\r\n');
                }
                ftpClient.ascii(function (err) {
                    if (err) {
                        return deferred.reject(err);
                    }
                    ftpClient.site('FILETYPE=JES', function (err1) {
                        if (err1) {
                            return deferred.reject(err1);
                        }
                        ftpClient.put(Buffer.from(jclText), 'PLACEHOL', function (err2, text) {
                            if (err2) {
                                deferred.reject(err2);
                            }
                            else {
                                var matched = text.match(/JES as (\w+)\s/i);
                                if (!matched || matched.length !== 2) {
                                    deferred.reject(new Error('Failed to submit jcl, job id not found'));
                                }
                                else {
                                    deferred.resolve(matched[1]);
                                }
                            }
                        });
                    });
                });
                return [2 /*return*/, deferred.promise];
            });
        });
    };
    /**
     * Convert allocate params object to name/value pair string.
     */
    ZosAccessor.prototype.allocateParamsToString = function (allocateParams) {
        if (allocateParams === undefined) {
            return '';
        }
        if (typeof allocateParams === 'string') {
            return allocateParams;
        }
        var params = [];
        // allocateParams is an object
        for (var key in allocateParams || {}) {
            if (!allocateParams.hasOwnProperty(key)) {
                continue;
            }
            if (allocateParams[key] === true) {
                params.push(key.toUpperCase());
            }
            else {
                params.push(key.toUpperCase() + '=' + allocateParams[key]);
            }
        }
        return params.join(' ');
    };
    /**
     * Check the length of jobname and job owner. If it longer than 8 characters throw an error.
     *
     * @param jobname the jobname or prefix
     * @param owner the owner of job
     */
    ZosAccessor.prototype.checkJobNameAndOwner = function (jobname, owner) {
        if ((jobname && jobname.length > 8) || (owner && owner.length > 8)) {
            throw new Error("Value of prefix or owner is not valid. It is longer than 8 characters.");
        }
    };
    return ZosAccessor;
}());
exports.ZosAccessor = ZosAccessor;
var USSEntry_2 = require("./interfaces/USSEntry");
exports.FileType = USSEntry_2.FileType;
